@@ -20,7 +20,7 @@
 //!     grid.set(Entity::Cell(0, 1), Settings::new().text("0-1"));
 //!     grid.set(Entity::Cell(1, 0), Settings::new().text("1-0"));
 //!     grid.set(Entity::Cell(1, 1), Settings::new().text("1-1"));
-//!     
+//!
 //!     let expected = concat!(
 //!         "+---+---+\n",
 //!         "|0-0|0-1|\n",
@@ -545,18 +545,8 @@ pub enum Alignment {
 }
 
 impl Alignment {
-    #[cfg(not(feature = "color"))]
     fn align(&self, text: &str, length: usize) -> String {
-        match self {
-            Alignment::Center => format!("{: ^1$}", text, length),
-            Alignment::Left => format!("{: <1$}", text, length),
-            Alignment::Right => format!("{: >1$}", text, length),
-        }
-    }
-
-    #[cfg(feature = "color")]
-    fn align(&self, text: &str, length: usize) -> String {
-        let mut diff = length - string_width(text);
+        let diff = length - string_width(text);
         match self {
             Alignment::Left => format!("{text}{space}", space = " ".repeat(diff), text = text),
             Alignment::Right => format!("{space}{text}", space = " ".repeat(diff), text = text),
@@ -680,20 +670,14 @@ fn string_width(text: &str) -> usize {
 
 #[cfg(feature = "color")]
 fn string_width(text: &str) -> usize {
-    // console::strip_ansi_codes(text)
     let b = strip_ansi_escapes::strip(text.as_bytes()).unwrap();
     let s = std::str::from_utf8(&b).unwrap();
-    let x = s
-        .lines()
-        .map(|line| line.chars().count())
-        .max()
-        .unwrap_or_else(|| 0);
-    x
+    real_string_width(&s)
 }
 
 fn real_string_width(text: &str) -> usize {
     text.lines()
-        .map(|line| line.chars().filter(|c| !c.is_control()).count())
+        .map(|line| textwrap::core::display_width(line))
         .max()
         .unwrap_or_else(|| 0)
 }
@@ -896,6 +880,37 @@ mod tests {
         )
     }
 
+    #[test]
+    #[ignore = "I am not sure what is the right behaiviour here"]
+    fn hieroglyph_handling() {
+        let mut grid = Grid::new(1, 2);
+        grid.set(Entity::Cell(0, 0), Settings::new().text("哈哈"));
+        grid.set(Entity::Cell(0, 1), Settings::new().text("哈"));
+        let s = grid.to_string();
+        assert_eq!(
+            s,
+            "+----+--+\n\
+             |哈哈  |哈 |\n\
+             +----+--+\n"
+        )
+    }
+
+    #[test]
+    #[ignore = "I am not sure what is the right behaiviour here"]
+    fn hieroglyph_multiline_handling() {
+        let mut grid = Grid::new(1, 2);
+        grid.set(Entity::Cell(0, 0), Settings::new().text("哈哈"));
+        grid.set(Entity::Cell(0, 1), Settings::new().text("哈\n哈"));
+        let s = grid.to_string();
+        assert_eq!(
+            s,
+            "+----+--+\n\
+             |哈哈  |哈 |\n\
+             |    |哈 |\n\
+             +----+--+\n"
+        )
+    }
+
     #[cfg(feature = "color")]
     #[test]
     fn colored_string_width_test() {
@@ -905,4 +920,31 @@ mod tests {
         assert_eq!(string_width("\u{1b}[34m0\u{1b}[0m"), 1);
         assert_eq!(string_width(&"0".red().to_string()), 1);
     }
+
+    #[test]
+    fn string_width_emojie_test() {
+        // ...emojis such as “joy”, which normally take up two columns when printed in a terminal
+        // https://github.com/mgeisler/textwrap/pull/276
+        assert_eq!(string_width("🎩"), 2);
+        assert_eq!(string_width("Rust 💕"), 7);
+        assert_eq!(string_width("Go 👍\nC 😎"), 5);
+    }
+
+    #[test]
+    fn aligment_test() {
+        assert_eq!(Alignment::Right.align("AAA", 4), " AAA");
+        assert_eq!(Alignment::Left.align("AAA", 4), "AAA ");
+        assert_eq!(Alignment::Center.align("AAA", 4), "AAA ");
+        assert_eq!(Alignment::Center.align("🎩", 4), " 🎩 ");
+        assert_eq!(Alignment::Center.align("🎩", 3), "🎩 ");
+        #[cfg(feature = "color")]
+        {
+            use colored::Colorize;
+            let text = "Colored Text".red().to_string();
+            assert_eq!(
+                Alignment::Center.align(&text, 15),
+                format!(" {}  ", text),
+            );
+        }
+    }
 }
